package org.opengroup.osdu.notification.errors;

import org.checkerframework.checker.units.qual.A;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppError;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.context.request.ServletWebRequest;
import org.springframework.web.context.request.WebRequest;

import javax.servlet.http.HttpServletRequest;
import javax.xml.bind.ValidationException;
import java.io.IOException;
import java.nio.file.AccessDeniedException;
import java.util.Objects;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(MockitoJUnitRunner.class)
public class SpringExceptionMapperTest {

    @InjectMocks
    private SpringExceptionMapper springExceptionMapper;

    @Mock
    private JaxRsDpsLog log;

    @Mock
    private HttpServletRequest servletRequest;

    @Mock
    private WebRequest request;

    @Before
    public void setUp() {
        MockitoAnnotations.initMocks(SpringExceptionMapperTest.this);
    }

    @Test
    public void handleAppException() {
        AppException appException = new AppException(400, "Sample app exception reason", "Sample app exception message");
        appException.setOriginalException(new NullPointerException());
        ResponseEntity<Object> actual = springExceptionMapper.handleAppException(appException);
        AppError appError = (AppError) actual.getBody();
        assertNotNull(actual);
        assertEquals(400, Objects.requireNonNull(appError).getCode());
        assertEquals("Sample app exception reason", appError.getReason());
        assertEquals("Sample app exception message", appError.getMessage());
    }


    @Test
    public void handleValidationException() {
        ValidationException validationException = new ValidationException("Sample Validation Exception");
        ResponseEntity<Object> actual = springExceptionMapper.handleValidationException(validationException);
        assertNotNull(actual);
        AppError appError = (AppError) actual.getBody();
        assertEquals(400,appError.getCode());
        assertEquals("Sample Validation Exception",appError.getMessage());
        assertEquals("Bad request",appError.getReason());



    }

    @Test
    public void handleAccessDeniedException() {
        AccessDeniedException accessDeniedException = new AccessDeniedException("Sample Access Denied Exception");
        ResponseEntity<Object> actual = springExceptionMapper.handleAccessDeniedException(accessDeniedException,request);
        assertNotNull(actual);
        AppError appError = (AppError) actual.getBody();
        assertEquals(401,appError.getCode());
        assertEquals("Sample Access Denied Exception",appError.getMessage());
        assertEquals("Unauthorized",appError.getReason());

    }

    @Test
    public void handleGeneralException() {
        Exception exception = new Exception("A General Exception");
        ResponseEntity<Object> actual = springExceptionMapper.handleGeneralException(exception);
        AppError appError = (AppError) actual.getBody();
        assertNotNull(appError);
        assertEquals(500,appError.getCode());
        assertEquals("An unknown error has occurred.",appError.getMessage());
        assertEquals("Server Error",appError.getReason());


    }

    @Test
    public void handleIOException_connectionClosed() {
        IOException ioException = new IOException("Sample IO Exception due to broken pipe");
        ResponseEntity<Object> actual = springExceptionMapper.handleIOException(ioException);
        assertEquals(actual,null);

    }

    @Test
    public void handleIOExceptionTest() {
        IOException ioException = new IOException("Sample IO Exception");
        ResponseEntity<Object> actual = springExceptionMapper.handleIOException(ioException);
        assertNotNull(actual);
        AppError appError = (AppError) actual.getBody();
        assertEquals("Unknown error",appError.getReason());
        assertEquals("Sample IO Exception",appError.getMessage());
        assertEquals(503,appError.getCode());

    }
}